class ContractsController < ApplicationController
  layout "cost_admin"

  helper :queries
  include QueriesHelper
  include ContractsHelper
  helper :contracts_queries
  include ContractsQueriesHelper

  accept_api_auth :index, :show, :create, :update, :destroy

  def index
    use_session = !request.format.csv?
    retrieve_query(ContractQuery, use_session)

    if @query.valid?
      respond_to do |format|
        format.html {
          @contract_count = @query.contract_count
          @contract_pages = Paginator.new @contract_count, per_page_option, params["page"]
          @contracts = @query.contracts(:offset => @contract_pages.offset, :limit => @contract_pages.per_page)
          render :layout => !request.xhr?
        }
        format.api {
          @offset, @limit = api_offset_and_limit
          @query.column_names = %w(author)
          @contract_count = @query.contract_count
          @contracts = @query.contracts(:offset => @offset, :limit => @limit)
        }
      end
    else
      respond_to do |format|
        format.html { render :layout => !request.xhr? }
        format.any(:atom, :csv, :pdf) { head 422 }
        format.api { render_validation_errors(@query) }
      end
    end
  rescue ActiveRecord::RecordNotFound
    render_404
  end

  def show
    @contract = Contract.find_by_id(params[:id])
    unless @contract && @contract.visible?
      render_404
      return
    end

    respond_to do |format|
      format.html {
        render :layout => "cost_admin"
      }
      format.api
    end
  end

  def new
    @contract = Contract.new()
    @contract.safe_attributes = params[:contract]
    @projects = Project.active.all
    @users = User.active.all
    @counter_parties = Entity.all
    respond_to do |format|
      format.html { render :action => "new", :layout => !request.xhr? }
    end
  end

  def create
    @projects = Project.active.all
    @users = User.active.all
    @counter_parties = Entity.all
    @contract = Contract.new()
    @contract.safe_attributes = params[:contract]
    @contract.created_by = User.current

    if @contract.save
      respond_to do |format|
        format.html {
          flash[:notice] = l(:notice_contract_successful_create, :id => view_context.link_to(@contract.contract_identifier, contract_path(@contract)))
          if params[:continue]
            redirect_to new_contract_path
          else
            redirect_to edit_contract_path(@contract)
          end
        }
        format.api { render :action => "show", :status => :created, :location => contract_url(@contract) }
      end
    else
      respond_to do |format|
        format.html { render :action => "new" }
        format.api { render_validation_errors(@contract) }
      end
    end
  end

  def edit
    @contract = Contract.find_by_id(params[:id])
    @projects = Project.active.all
    @users = User.active.all
    @counter_parties = Entity.all
  end

  def update
    @contract = Contract.find_by_id(params[:id])
    @contract.safe_attributes = params[:contract]
    @contract.updated_by = User.current

    if @contract.save
      respond_to do |format|
        format.html {
          flash[:notice] = l(:notice_successful_update)
          redirect_to_referer_or edit_contract_path(@contract)
        }
        format.api { render_api_ok }
      end
    else
      @projects = Project.active.all
      @users = User.active.all

      respond_to do |format|
        format.html { render :action => :edit }
        format.api { render_validation_errors(@contract) }
      end
    end
  end

  def destroy
    @contract = Contract.find_by_id(params[:id])
    @contract.destroy
    respond_to do |format|
      format.html { redirect_back_or_default(contracts_path) }
      format.api { render_api_ok }
    end
  end
end
