class ContractImport < Import
  def self.menu_item
    :Contracts
  end

  def self.authorized?(user)
    user.allowed_to?(:edit_contracts, nil, :global => true)
  end

  # Returns the objects that were imported
  def saved_objects
    object_ids = saved_items.pluck(:obj_id)
    objects = Contract.where(:id => object_ids).order(:id).preload(:user, :project, :counter_party)
  end

  def mappable_custom_fields
    # there is no feature to make custom field in Contract
    []
  end

  def allowed_target_projects
    Project.allowed_to(user, :edit_contracts).order(:lft)
  end

  def project
    project_id = mapping["project_id"].to_i
    allowed_target_projects.find_by_id(project_id) || allowed_target_projects.first
  end

  def mappable_core_fields_for_ss
    [
      "exclude",
      "contract_identifier",
      "user",
      "project",
      "contract_type",
      "counter_party",
      "date_valid_from",
      "date_valid_to",
      "reward_unit",
      "unit_price",
      "original_currency",
      "original_unit_price",
      "exchange_rate",
    ]
  end

  def mappable_custom_fields_for_ss
    # there is no feature to make custom field for Contract
    []
  end

  # concatenates core fields and custom fields as field names
  def mappable_fields_for_ss
    mappable_fields = self.mappable_core_fields_for_ss
    mappable_fields
  end

  # makes local language version of fields list for select
  def attribute_options_for_ss
    attributes = []
    core_fields = self.mappable_core_fields_for_ss
    core_fields.each do |field|
      field_symbol_str = "field_" + field.sub(/_ids?\z/, "")
      local_attribute_name = l(field_symbol_str.to_sym)
      if local_attribute_name
        attributes << local_attribute_name
      end
    end

    i = -1
    attributes.map { |h| [h, i += 1] }
  end

  # validates each row of CSV/SS by tentatively build_object
  def validate_objects_for_ss
    self.settings["errors"] = {}
    self.settings["ignores"] = {}
    self.settings["excludes"] = {}

    read_items do |row, position|
      errors_per_position = {}
      ignores_per_position = {}

      if exclude = row_value(row, "exclude")
        self.settings["excludes"].merge!({ position => true })
        next
      end

      contract = Contract.new
      ## contract_identifier: mandatory
      attributes = { "contract_identifier" => row_value(row, "contract_identifier") }
      contract.send :safe_attributes=, attributes, user
      unless contract.contract_identifier && contract.contract_identifier == row_value(row, "contract_identifier")
        errors_per_position.merge!({ "contract_identifier" => mapping["contract_identifier"] })
      end

      ## contract_type: mandatory
      contract_type = row_value(row, "contract_type")
      contract_type_options = Contract.valid_contract_type_options.collect { |v, lv| [v, l(lv)] }
      unless contract_type_options.select { |cto| cto[0] == contract_type || cto[1] == contract_type }.any?
        errors_per_position.merge!({ "contract_type" => mapping["contract_type"] })
      end

      ## user: mandatory
      user_name = row_value(row, "user")
      if contract_user = User.detect_by_keyword(User.active.all, user_name)
        attributes = { "user_id" => contract_user.id }
        contract.send :safe_attributes=, attributes, user

        unless contract.user && contract.user.name == row_value(row, "user")
          errors_per_position.merge!({ "user" => mapping["user"] })
        end
      end

      # project: non-mandatory
      if project_name = row_value(row, "project")
        if project = Project.find_by_name(project_name)
          attributes = { "project_id" => project.id }
          contract.send :safe_attributes=, attributes, user

          unless contract.project && contract.project.name == row_value(row, "project")
            errors_per_position.merge!({ "project" => mapping["project"] })
          end
        elsif project == "nil"
          contract.project_id = nil
          errors_per_position.merge!({ "project" => mapping["project"] }) unless contract.project == nil
        else
          errors_per_position.merge!({ "project" => mapping["project"] })
        end
      end
      # counter_party: mandatory
      counter_party_name = row_value(row, "counter_party")
      if counter_party = Entity.detect_by_keyword(Entity.all, counter_party_name)
        attributes = { "counter_party_id" => counter_party.id }
        contract.send :safe_attributes=, attributes, user

        unless contract.counter_party && (contract.counter_party.name == row_value(row, "counter_party") || contract.counter_party.short_name == row_value(row, "counter_party"))
          errors_per_position.merge!({ "counter_party" => mapping["counter_party"] })
        end
      else
        errors_per_position.merge!({ "counter_party" => mapping["counter_party"] })
      end

      # date_valid_from: mandatory
      if date_valid_from = row_date(row, "date_valid_from")
        attributes = { "date_valid_from" => date_valid_from }
        contract.send :safe_attributes=, attributes, user

        unless contract.date_valid_from && contract.date_valid_from == row_date(row, "date_valid_from")
          errors_per_position.merge!({ "date_valid_from" => mapping["date_valid_from"] })
        end

        if c = contract.user.valid_contract
          if range = c.date_valid_from..c.date_valid_to
            if range.cover?(date_valid_from)
              errors_per_position.merge!({ "date_valid_from" => mapping["date_valid_from"] })
            end
          end
        end
      else
        errors_per_position.merge!({ "date_valid_from" => mapping["date_valid_from"] })
      end

      # date_valid_to: mandatory
      if date_valid_to = row_date(row, "date_valid_to")
        attributes = { "date_valid_to" => date_valid_to }
        contract.send :safe_attributes=, attributes, user

        unless contract.date_valid_to && contract.date_valid_to == row_date(row, "date_valid_to")
          errors_per_position.merge!({ "date_valid_to" => mapping["date_valid_to"] })
        end

        if c = contract.user.valid_contract
          if range = c.date_valid_from..c.date_valid_to
            if range.cover?(date_valid_to)
              errors_per_position.merge!({ "date_valid_to" => mapping["date_valid_to"] })
            end
          end
        end
      else
        errors_per_position.merge!({ "date_valid_to" => mapping["date_valid_to"] })
      end
      # reward_unit: mandatory
      reward_unit = row_value(row, "reward_unit")
      reward_unit_options = Contract.valid_reward_unit_options.collect { |v, lv| [v, l(lv)] }
      unless reward_unit_options.select { |ruo| ruo[0] == reward_unit || ruo[1] == reward_unit }.any?
        errors_per_position.merge!({ "reward_unit" => mapping["reward_unit"] })
      end

      # unit_price: mandatory
      unit_price = row_value(row, "unit_price").to_f
      attributes = { "unit_price" => unit_price }
      contract.send :safe_attributes=, attributes, user
      unless contract.unit_price && contract.unit_price == row_value(row, "unit_price").to_f
        errors_per_position.merge!({ "unit_price" => mapping["unit_price"] })
      end

      # original_currency: non-mandatory
      # original_unit_price: non-mandatory
      # exchange_rate: non-mandatory

      ## finally, merge errors and ignores per line into all errors
      if errors_per_position.count > 0
        self.settings["errors"].merge!({ position => errors_per_position })
      else
        self.settings["errors"].delete(position)
      end
      if ignores_per_position.count > 0
        self.settings["ignores"].merge!({ position => ignores_per_position })
      else
        self.settings["ignores"].delete(position)
      end
    end
  end

  private

  def build_object(row, item)
    contract = Contract.new()

    if update?
      contract = Contract.find(row_value(row, "contract_identifier"))
    end

    attributes = {}

    attributes["contract_identifier"] = row_value(row, "contract_identifier")

    user_name = row_value(row, "user")
    if contract_user = User.detect_by_keyword(User.active.all, user_name)
      attributes["user_id"] = contract_user.id
    end

    attributes["project_id"] = project.id if project = Project.find_by_name(row_value(row, "project"))

    if contract_type = contract.valid_contract_type_options.select { |v, lv|
      v == row_value(row, "contract_type") || l(lv) == row_value(row, "contract_type")
    }
      attributes["contract_type"] = contract_type.first.first
    end

    if counter_party = Entity.detect_by_keyword(Entity.all, row_value(row, "counter_party"))
      attributes["counter_party_id"] = counter_party.id
    end

    attributes["date_valid_from"] = row_date(row, "date_valid_from")
    attributes["date_valid_to"] = row_date(row, "date_valid_to")

    if reward_unit = contract.valid_reward_unit_options.select { |v, lv|
      v == row_value(row, "reward_unit") || l(lv) == row_value(row, "reward_unit")
    }
      attributes["reward_unit"] = reward_unit.first.first
    end
    attributes["unit_price"] = row_value(row, "unit_price")

    contract.send :safe_attributes=, attributes, user
    contract
  end
end
