class Expense < ActiveRecord::Base
  include Redmine::SafeAttributes
  acts_as_customizable
  acts_as_watchable

  belongs_to :project
  belongs_to :created_by, :class_name => "User"
  belongs_to :updated_by, :class_name => "User"

  validates_presence_of :project, :expense_identifier, :date_expensed
  validates_uniqueness_of :expense_identifier, :if => Proc.new { |expense|
                         expense.expense_identifier_changed? && expense.expense_identifier.present?
                       }, :case_sensitive => false

  scope :visible, lambda { |*args|
    where(Expense.visible_condition(args.shift || User.current, *args))
  }

  scope :sorted, lambda { order(*Expense.fields_for_order_statement) }

  scope :like, lambda { |q|
    q = q.to_s
    if q.blank?
      where({})
    else
      pattern = "%#{q}%"
      sql = +"LOWER(#{table_name}.description) LIKE LOWER(:p)"
      params = { :p => pattern }

      where(sql, params)
    end
  }

  safe_attributes(
    "project_id",
    "expense_identifier",
    "date_expensed",
    "amount",
    "description",
  )

  def visible?(user = User.current)
    user.allowed_to?(:view_expenses, nil) || user.admin?
  end

  def editable?(user = User.current)
    user.allowed_to?(:edit_expenses, nil) || user.admin?
  end

  def deletable?
    user.allowed_to?(:edit_expenses, nil) || user.admin?
  end

  def self.visible_condition(user, options = {})
    sql = "1=0"
    sql = "1=1" if user.allowed_to?(:view_expenses, nil, :global => true) || user.admin?
    sql
  end

  def self.fields_for_order_statement(table = nil)
    table ||= table_name
    columns = ["name"]
    columns.uniq.map { |field| "#{table}.#{field}" }
  end

  # Returns a string of css classes that apply to the contract
  def css_classes(user = User.current)
    s = +"contract"
    s
  end
end
