Rails.logger.info "Starting Redmine Cost Administration Plugin"

require "redmine_cost_admin/project_patch"
require "redmine_cost_admin/user_patch"
require "redmine_cost_admin/issue_patch"
require "redmine_cost_admin/users_controller_patch"
require "redmine_cost_admin/queries_controller_patch"
require "redmine_cost_admin/users_helper_patch"
require "redmine_cost_admin/imports_helper_patch"
require "redmine_cost_admin/version_patch"
require "redmine_cost_admin/issues_view_hooks"

Redmine::Plugin.register :redmine_cost_admin do
  begin
    name "Redmine Cost Administration plugin"
    author "HARAGUCHI, Toru"
    description "Provides personel\'s cost allocation, budgets and expences accounting"
    version "0.0.1"
    # url "http://example.com/path/to/plugin"
    # author_url "http://example.com/about"

    menu :top_menu, :redmine_cost_admin, { controller: "cost_admin", action: "index" },
      :caption => :label_contract_management, :if => Proc.new { |p|
        User.current.admin? ||
        User.current.allowed_to?(:edit_entities, nil, { :global => true }) ||
        User.current.allowed_to?(:view_entities, nil, { :global => true }) ||
        User.current.allowed_to?(:edit_contracts, nil, { :global => true }) ||
        User.current.allowed_to?(:view_contracts, nil, { :global => true })
      }
    menu :application_menu, :budgets, { controller: "budgets", action: "index" },
         caption: :label_budgets, after: :time_entries,
         :if => Proc.new { |p|
           User.current.admin? ||
           User.current.allowed_to?(:edit_budgets, p) ||
           User.current.allowed_to?(:view_budgets, p)
         }
    menu :application_menu, :expenses, { controller: "expenses", action: "index" },
         caption: :label_expenses, after: :budgets,
         :if => Proc.new { |p|
           User.current.admin? ||
           User.current.allowed_to?(:edit_expenses, p) ||
           User.current.allowed_to?(:view_expenses, p)
         }
    menu :project_menu, :budgets, { controller: "budgets", action: "index" },
         caption: :label_budgets, after: :time_entries, param: :project_id,
         :if => Proc.new { |p|
           User.current.admin? ||
           User.current.allowed_to?(:edit_budgets, p) ||
           User.current.allowed_to?(:view_budgets, p)
         }
    menu :project_menu, :expenses, { controller: "expenses", action: "index" },
         caption: :label_expenses, after: :budgets, param: :project_id,
         :if => Proc.new { |p|
           User.current.admin? ||
           User.current.allowed_to?(:edit_expenses, p) ||
           User.current.allowed_to?(:view_expenses, p)
         }

    Redmine::MenuManager.map :cost_admin_menu do |menu|
      menu.push :entities, { :controller => "entities" }, :caption => :label_entity_plural
      menu.push :contracts, { :controller => "contracts" }, :caption => :label_contract_plural
    end

    settings :default => {
               "contract_list_defaults" => [
                 "contract_identifier",
                 "contract_type",
                 "user",
                 "project",
                 "counter_party",
                 "date_valid_from",
                 "date_valid_to",
                 "reward_unit",
                 "unit_price",
                 "original_currency",
                 "original_unit_price",
                 "exchange_rate",
               ],
               "budget_list_defaults" => [
                 "project",
                 "budget_identifier",
                 "date_valid_from",
                 "date_valid_to",
                 "amount",
                 "description",
               ],
               "expense_list_defaults" => [
                 "project",
                 "expense_identifier",
                 "date_expensed",
                 "amount",
                 "description",
               ],
             }, :partial => "settings/redmine_cost_admin"

    project_module :cost_admin do
      permission :edit_budgets, budgets: %i[new create edit update destroy move]
      permission :view_budgets, budgets: %i[index show load]
      permission :edit_expenses, expenses: %i[new create edit update destroy move]
      permission :view_expenses, expenses: %i[index show load]
      permission :edit_entities, entities: %i[new create edit update destroy move]
      permission :view_entities, entities: %i[index show load]
      permission :edit_contracts, contracts: %i[new create edit update destroy move]
      permission :view_contracts, contracts: %i[index show load]
      permission :view_total_cost, {}
    end
  rescue ::Redmine::PluginRequirementError => e
    raise ::Redmine::PluginRequirementError.new(issue_template_version_message(e.message)) # rubocop:disable Style/RaiseArgs
  end
end
